import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import levelObjects
                      
                      
class StateGame(fsmMgr.State):
    def __init__(self, parentState, data):
        #parentState is None, right?
        self.angle=0
        self.power=0
        self.accuracy=0

        self.setLevel(1)
        
    def nextLevel(self):
        self.setLevel(self.level+1)

    def setLevel(self, levelNum):
        self.level=levelNum

        self.physicsObjects=levelObjects.getObjects(self.level)

        self.currentPlanet=self.physicsObjects[0]

        self.levelTime=0.0
        
        

    def update(self, ms):
        self.levelTime += ms/1000.0

        for p in self.physicsObjects:
            parentName=p.get("orbits",None)
            if not (parentName is None):
                for parent in self.physicsObjects:
                    if parent["name"]==parentName:
                        orbitRate=p.get("orbitRate")
                        timeIntoOrbit=math.fmod(self.levelTime,abs(orbitRate))
                        fracAroundOrbit=timeIntoOrbit/orbitRate+p["orbitOffset"]
                        angle=math.pi*2.0*fracAroundOrbit
                        orbRadius=p["orbitRadius"]
                        pcx,pcy=parent["center"]
                        p["center"]=(pcx+math.cos(angle)*orbRadius,
                                     pcy+math.sin(angle)*orbRadius)

        

        
    def draw(self):
        for p in self.physicsObjects:

            cx,cy=p["center"]
            r=p["radius"]
            color=p["color"]

            glColor3f(*color)
            glBegin(GL_LINE_LOOP)

            NUM_STEPS=30
            inc_angle=math.pi*2.0/float(NUM_STEPS)
            
            for i in range(NUM_STEPS):
                a=i*inc_angle
                dx=math.cos(a)*r
                dy=math.sin(a)*r

                glVertex2f(cx+dx,cy+dy)
            glEnd()

            if p.get("target",False):
                off=r*0.65
                glBegin(GL_LINES)
                glVertex2f(cx-off,cy-off)
                glVertex2f(cx+off,cy+off)
                glVertex2f(cx-off,cy+off)
                glVertex2f(cx+off,cy-off)
                glEnd()

            if p.get("gaseous",False):
                wiggleData=[(.1, .80),
                            (.2, .72),
                            (.3, .70),
                            (.4, .67),
                            (.5, .69),
                            (.6, .74),
                            (.7, .80),
                            (.8, .80),
                            (.9, .74),
                            (.95, .71)]

                for deltaY in [0,-.4]:
                    glBegin(GL_LINE_STRIP)
                    for wx,wy in wiggleData:
                        s=2*r
                        glVertex(cx-r+s*wx,
                                 cy-r+s*(wy+deltaY))

                    glEnd()
                    
                
                
                
            

thisLevel=StateGame(None, None)
