import sys
import os
import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import screenInfo
import level1
import stateAim
import stateEdit
import fsmMgr

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(0,800,0,600)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():
    glEnable(GL_TEXTURE_2D)
    glShadeModel(GL_SMOOTH)
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glClearDepth(1.0)
    glEnable(GL_DEPTH_TEST)
    glEnable(GL_CULL_FACE)
    glDepthFunc(GL_LEQUAL)
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
    glPointSize(3.0)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glLineWidth(2.5)
    glEnable(GL_LINE_SMOOTH)
    glHint(GL_LINE_SMOOTH_HINT,GL_FASTEST)


def readMouse():
    global mouseX,mouseY

    mouseX,mouseY=pygame.mouse.get_pos()

def d2r(degrees):
    return degrees*math.pi*2.0/360.0
    
def update(ms):
    fsmMgr.update(ms)

def draw():
    glClear(GL_COLOR_BUFFER_BIT)
    fsmMgr.draw()

def handleKey(k):
    global controlKey
    controlKey=''

    if fsmMgr.handleKey(k):
        return
    
    if k == K_LEFT:
        controlKey='left'
    elif k == K_RIGHT:
        controlKey='right'

    elif k==K_s:
        level1.thisLevel.nextLevel()

    elif k==K_e:
        fsmMgr.pushState(stateEdit.StateEdit(None))

def handleMouseButton(bDown, button):
    fsmMgr.handleMouseButton(bDown, button)
    

def main(bFullScreen, sz, levelSkip):
    video_flags = OPENGL|DOUBLEBUF
    if bFullScreen:
        video_flags = video_flags|FULLSCREEN

    print "about to init"
    pygame.init()
    surface = pygame.display.set_mode(sz, video_flags)

    print "about to resize"
    resize(sz)


    print "about to set screenInfo"
    screenInfo.gWindowX,screenInfo.gWindowY=sz
    screenInfo.gScreenX,screenInfo.gScreenY=(800,600)

    pygame.display.set_caption("WTIPD - BDG PyWeek4")
    pygame.mouse.set_visible(False)
    pygame.mouse.set_pos((sz[0]/2,sz[1]/2))
    init()


    if levelSkip<1:
        levelSkip=1
        
    level1.thisLevel.setLevel(levelSkip)
    fsmMgr.pushState(level1.thisLevel)
    fsmMgr.pushState(stateAim.StateAim(None))


    frames = 0
    ticks = pygame.time.get_ticks()
    myClock=pygame.time.Clock()

    myClock.tick()

    timeToQuit=False
    isActive=True
    
    while 1:
        ms=myClock.tick()

        for event in pygame.event.get():
            if (event.type == QUIT or
                (event.type == KEYDOWN and event.key == K_ESCAPE) or
                (event.type == KEYDOWN and event.key == K_F4)):
                timeToQuit=True
                break

            if event.type == KEYDOWN:
                handleKey(event.key)

            elif event.type == MOUSEBUTTONDOWN:
                handleMouseButton(True, event.button)
            elif event.type == MOUSEBUTTONUP:
                handleMouseButton(False, event.button)

            elif event.type == KEYUP:
                global controlKey
                controlKey = ''
            elif event.type==ACTIVEEVENT:
                if event.gain==1:
                    pygame.event.set_grab(True)
                    pygame.mouse.set_visible(False)
                    isActive=True
                else:
                    pygame.event.set_grab(False)
                    pygame.mouse.set_visible(True)
                    isActive=False
                    
                
        if timeToQuit:
            break

        if isActive:
            readMouse()

        if isActive:
            update(ms)
        draw()
        pygame.display.flip()
        frames = frames+1


    print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__':
    level=1
    if len(sys.argv)>1:
        level=int(sys.argv[1])
    main(False, (800,600), level)
    #main(False, (400,300))
    #main(True, (800,600))
