import fsmMgr

import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import mouseInfo
import stateAim

GRAV_CONST = 800000

LOC_INCREMENT = 0.1
DRAW_ACCELERATION = False

class StateFlight(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.angle=0 #east, I guess

        self.curPlan={"radius":40,
                      "center":(80,80)}

        self.flightTime=0
        self.locs=[]
        self.exitMouseDown=False
        
    def update(self, dt):
        self.parent.update(dt)

        dt=dt/1000.0
        self.flightTime += dt

        # use simplectic integrator:
        # the idea is to update velocity first, then position
        ax,ay=self.calcAcceleration()
        parent=self.parent
        
        parent.packageVX += ax*dt
        parent.packageVY += ay*dt
        parent.packageX += parent.packageVX*dt
        parent.packageY += parent.packageVY*dt

        p=self.landed()

        if p:
            resetZoom()
            if p.get("target",False):
                fsmMgr.popState()
                self.parent.nextLevel()
                fsmMgr.pushState(stateAim.StateAim(None))
            elif p.get("gaseous",False):
                # oh no!
                fsmMgr.popState()
                fsmMgr.pushState(stateAim.StateAim(None))            

            else:
                parent.currentPlanet=p
                fsmMgr.popState()
                fsmMgr.pushState(stateAim.StateAim(None))            

        if (parent.packageX>1600 or
            parent.packageX<-800 or
            parent.packageY>1500 or
            parent.packageY<-900):
            resetZoom()
            fsmMgr.popState()
            fsmMgr.pushState(stateAim.StateAim(None))

        self.logPosition()

    def logPosition(self):
        doLog=False
        if len(self.locs)==0:
            doLog=True
        else:
            lastTime,pos=self.locs[-1]
            if self.flightTime-lastTime>=LOC_INCREMENT:
                doLog=True

        if doLog:
            self.locs.append((self.flightTime,
                              (self.parent.packageX,
                               self.parent.packageY)))
            

    def handleMouseButton(self, bDown, buttonIndex):
        #trigger this at the beginning of the click

        if bDown==True:
            self.exitMouseDown=True
        elif self.exitMouseDown:
            resetZoom()
            fsmMgr.popState()
            fsmMgr.pushState(stateAim.StateAim(None))
            return True
        return False
    
    def draw(self):
        px=self.parent.packageX
        py=self.parent.packageY

        zoomOut(px,py)
        
        # first, let our parent draw the map
        self.parent.draw()

        glColor3f(1,1,1)
        glBegin(GL_LINE_LOOP)
        glVertex(px-5,py)
        glVertex(px,py-5)
        glVertex(px+5,py)
        glVertex(px,py+5)
        glEnd()

        if DRAW_ACCELERATION:
            glBegin(GL_LINES)
            glVertex(px,py)
            glVertex(px+self.ax*2,py+self.ay*2)
            glEnd()

        duration=4.0
        
        glColor3f(1.0, 0.5, 0.0)
        baseR=0.8
        baseG=0.7
        baseB=0.0
        
        for loc in self.locs:
            tm,pos=loc
            if self.flightTime-tm>duration:
                continue
            fade=(self.flightTime-tm)/duration
            bright=1.0-fade
            glColor3f(baseR*bright,
                      baseG*bright,
                      baseB*bright)

            px,py=pos
            glBegin(GL_LINE_LOOP)
            glVertex(px-3,py)
            glVertex(px,py-3)
            glVertex(px+3,py)
            glVertex(px,py+3)
            glEnd()

            


    def calcAcceleration(self):
        # return ax,ay

        #Force = -gMM/r^2

        #factor out the package's mass:

        # a = -gM/r^2

        ax=0
        ay=0

        NO_ACCEL_PERIOD=0.5
        GRACE_PERIOD=1.0

        if self.flightTime<NO_ACCEL_PERIOD:
            return ax,ay

        px=self.parent.packageX
        py=self.parent.packageY

        for obj in self.parent.physicsObjects:
            ox,oy=obj["center"]
            dx=ox-px
            dy=oy-py
            rSqr=dx*dx+dy*dy
            r=math.sqrt(rSqr)
            totalA=GRAV_CONST*obj["mass"]/rSqr

            oax=totalA*dx/r
            oay=totalA*dy/r

            ax+=oax
            ay+=oay

            #print obj["name"],oax,oay

        self.ax=ax
        self.ay=ay
        #print "total",ax,ay
        #print

        if self.flightTime<GRACE_PERIOD:
            r=(self.flightTime-NO_ACCEL_PERIOD)/(GRACE_PERIOD-NO_ACCEL_PERIOD)
            self.ax*=r
            self.ay*=r
        return ax,ay
        
    def landed(self):
        px=self.parent.packageX
        py=self.parent.packageY

        for obj in self.parent.physicsObjects:
            if obj==self.parent.currentPlanet and self.flightTime<1.0:
                continue
            ox,oy=obj["center"]
            dx=ox-px
            dy=oy-py
            rSqr=dx*dx+dy*dy
            objRad=obj["radius"]
            if rSqr<=objRad*objRad:
                return obj
        return None
        

def zoomOut(px, py):
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    minX=min(0,px-50)
    maxX=max(800, px+50)
    minY=min(0,py-50)
    maxY=max(600, py+50)

    centerX=(maxX+minX)/2
    centerY=(maxY+minY)/2

    wX=maxX-minX
    wY=maxY-minY

    aspect=4.0/3.0
    if wX>wY*aspect:
        #need to boost wY
        wY=wX/aspect
        minY=centerY-wY/2
        maxY=centerY+wY/2
    elif wY>wX/aspect:
        #need to boost wX
        wX=wY*aspect
        minX=centerX-wX/2
        maxX=centerX+wX/2
    
    gluOrtho2D(minX, maxX,
               minY, maxY)
               
               
               

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
        
def resetZoom():
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(0,800,0,600)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    
